#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, traceback, sys
import magic
import cgi
import tempfile
import shutil
import re
import gettext
from VideoEnPoche import config, webservices, packages
from VideoEnPoche.dal import Movie, Session
from log import VEPLog

from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

_ = gettext.gettext
gettext.textdomain ('videoenpoche')
gettext.bindtextdomain ('videoenpoche', '/usr/share/locale/')
gettext.bind_textdomain_codeset ('videoenpoche', 'UTF-8')
session = Session()

def process_add_movie_form(form, movie_path):
    VEPLog("videoenpoche-agentd", "AgentManager: Processing form")
    fields = {
        'title':  form['title'].value,
        'desc': form['desc'].value,
        'length': form['length'].value,
        'price': form['price'].value,
        'vat': form['vat'].value,
        'right_holders_share': form['right_holders_share'].value,
        'size': int(os.path.getsize(form['film'].filepath))/(1024*1024),
        'filename': movie_path,
        'poster_filename': os.path.basename(form['poster'].filepath),
        'license_id' : 1,
    }
    movie = None
    try:
        movie = session.query(Movie).get(int(form["editmovie_id"].value))
        m_path = os.path.join(config.get('datadir'), "movies", movie.filename)
        fields["id"] = movie.id
    except KeyError:
        pass
    movie=Movie(**fields)
    try:
        movie.bonus_desc = form['desc_bonus'].value
    except:
        movie.bonus_desc = None
    package = packages.Package(movie)
    try:
        pkg_name = form["package_name"].value
        package.package_path = packages.get_pkg_from_name(pkg_name).package_path
    except:
        pass
    package.make()
    package.post()


class DroopyFieldStorage(cgi.FieldStorage):
    """The file is created in the destination directory and its name is
    stored in the tmpfilename attribute.
    """

    def make_file(self, binary=None):
        temp = os.path.join (config.get('datadir'),"temp")
        fd, name = tempfile.mkstemp(dir=temp)
        self.tmpfile = os.fdopen(fd, 'w+b')
        self.tmpfilename = name
        return self.tmpfile


class WebHandler (BaseHTTPRequestHandler):
    def _send_file (self, filename, mimetype=None):
        # some special cases
        if not mimetype:
            if filename.endswith ('.css'):
                mimetype = "text/css"

        if not mimetype:
            ms = magic.open(magic.MAGIC_MIME)
            ms.load ()
            mimetype = ms.file (filename)

        self.send_response (200)
        self.send_header ('Content-Type', mimetype)
        self.end_headers ()


        f = open (filename, 'r')
        for l in f.readlines():
            l = re.sub (r"___((?:[^(___)]|\(|\))+)___", self._i18n, l)
            self.wfile.write (l)
        f.close ()

    def _i18n (self, matchobj):
        return _(matchobj.group(1))

    def do_GET (self):
        if self.path == '/':
            index = os.path.join (config.get('datadir'),
                                  config.get('web_theme'),
                                  'index.html')
            self._send_file (index, 'text/html')

        elif self.path.startswith ('/js/'):
            filename = os.path.join (config.get('datadir'),
                                     'js', self.path[4:])
            if os.path.isfile (filename):
                self._send_file (filename)
            else:
                self.send_error (404)
                self.end_headers ()

        elif self.path.startswith ('/webservices/'):
            tmp = self.path.split('/')[2]
            func = getattr (webservices, "ws_%s"%tmp)
            func (self)

        else:
            filename = os.path.join (config.get('datadir'),
                                     config.get('web_theme'),
                                     self.path[1:])
            if os.path.isfile (filename):
                self._send_file (filename)
            else:
                self.send_error (404)
                self.end_headers ()

    def do_POST(self):
        # Do some browsers /really/ use multipart ? maybe Opera ?
        if self.path == "/admin/add_movie/":
            self.add_movie()
        if self.path == "/admin/update_conf/":
            self.update_conf()

    def update_conf (self):
        form = cgi.FieldStorage (fp=self.rfile, headers=self.headers,
                                 environ={'REQUEST_METHOD': 'POST',
                                          'CONTENT_TYPE': self.headers['content-type']})
        a = {}
        for k in form.keys ():
            a[k] = form[k].value
        config.update (a)

        self.send_response(301)
        self.send_header('Location','/')
        self.end_headers()


    def add_movie(self):
        movies_path = os.path.join (config.get('datadir'),"movies")
        posters_path = os.path.join (config.get('datadir'),"default","images")
        self.log_message("Started file transfer")
        # -- Set up environment for cgi.FieldStorage
        env = {}
        env['REQUEST_METHOD'] = self.command
        if self.headers.typeheader is None:
            env['CONTENT_TYPE'] = self.headers.type
        else:
            env['CONTENT_TYPE'] = self.headers.typeheader
        form = DroopyFieldStorage(fp = self.rfile, environ = env)
        movie_path = form['film'].filename.decode('utf-8')
        movie_path = movie_path[0:movie_path.rfind(".")]
        movie_abs_path = os.path.join(movies_path, movie_path)
        if os.path.isdir(movie_abs_path):
            #Clean the movie path
            shutil.rmtree(movie_abs_path)
        os.mkdir(movie_abs_path)
        self.process_file(form['film'], movie_abs_path)
        try:
            self.process_file(form['bonus'], movie_abs_path)
        except:
            pass
        self.process_file(form['poster'], posters_path)
        try:
            process_add_movie_form(form, movie_path)
        except:
            print '-'*60
            traceback.print_exc(file=sys.stdout)
            print '-'*60
            err_500 = os.path.join (config.get('datadir'),
                                  config.get('web_theme'),
                                  '500.html')
            self._send_file (err_500, 'text/html')
            return
        VEPLog("videoenpoche-agentd","Building %s package."%form["title"].value)

        # -- Reply
        self.send_response(301)
        self.send_header('Location','/')
        self.end_headers()

    def process_file(self, fileitem, dest_path):
        filename = fileitem.filename.decode('utf-8')
        if filename == "":
            raise Exception("Empty filename")
        localpath = os.path.join(dest_path, filename).encode('utf-8')
        root, ext = os.path.splitext(localpath)
        i = 1
        # race condition, but hey...
        while (os.path.exists(localpath)):
            localpath = "%s-%d%s" % (root, i, ext)
            i = i+1
        if hasattr(fileitem, 'tmpfile'):
            # DroopyFieldStorage.make_file() has been called
            fileitem.tmpfile.close()
            shutil.move(fileitem.tmpfilename, localpath)
        else:
            # no temporary file, self.file is a StringIO()
            # see cgi.FieldStorage.read_lines()
            fout = file(localpath, 'wb')
            shutil.copyfileobj(fileitem.file, fout)
            fout.close()
        VEPLog("videoenpoche-agentd", "AgentManager: received : %s" % localpath)
        fileitem.filepath = localpath


def http_server_start ():
    port = config.get('server_port')

    try:
        os.environ["LANG"] = config.get('locale')
    except:
        os.environ["LANG"] = "C"

    server = HTTPServer (('', port), WebHandler)
    server.serve_forever ()




