# -*- conding: utf-8 -*-
from VideoEnPoche import config
from VideoEnPoche.jasonimporter import JasonImporter
from VideoEnPoche.log import VEPLog
from VideoEnPoche.package_handler import Package_handler
import urllib2
import os
import datetime as dt
import threading
import libtorrent as lt
import time
from VideoEnPoche.dal import Session, Revision, Movie
from VideoEnPoche.dal import Resolution, Category, License
from sqlalchemy import or_

class UpdatesManager(object):
    """This part is responsible for
     - fetching the updates from the server 
        (licenses, resolutions, movies, packages)
     - adding the movies to de download list
     - deleting the movies
    """
    pass

    def sync(self, entity, local, remote, l_fields, r_fields=None):
        """sync will synchronize a local entity with a remote list of objects
        according to the fields defined in l_fields and r_fields"""
        if r_fields is None: r_fields = l_fields
        fields = zip(l_fields, r_fields)
        for r in remote:
            try:
                l = (l for l in local if l.__getattribute__(l_fields[0]) == r[r_fields[0]]).next()
                for rf, lf in fields[1:]:
                    l.__setattr__(lf, r[rf])
                    self.session.add(l)
            except StopIteration:
                init_dict = {}
                for rf, lf in fields:
                    init_dict[lf] = r[rf]
                l = entity(**init_dict)
                self.session.add(l)
        for l in local:
            if not any(l.__getattribute__(l_fields[0]) == r[r_fields[0]] for r in remote):
                self.session.delete(l)


    def __init__(self, agent):
        super(UpdatesManager, self).__init__()
        self.agent = agent
        self.connector = JasonImporter(
                config.get('vep_server')+config.get('api_url'),
                config.get('login'), config.get('passwd'))
        self.session = agent.session
        self.package_handler = Package_handler(self.session)


    def run(self):
        self.update_resolutions()
        self.update_licenses()
        self.update_categories()
        self.update_movies()
        self.update_forced_seeds()

    @property
    def local_revision(self):
        nmbr = self.session.query(Revision.number).\
                    order_by(Revision.number.desc()).first()
        return (nmbr[0] if nmbr else 0)

    @local_revision.setter
    def local_revision(self, rev):
        VEPLog(__name__, "setting local revision to %s"%rev)
        self.session.add(Revision(number=rev))

    def update_resolutions(self):
        local = self.session.query(Resolution).all()
        remote = self.connector.get_resolutions()
        self.sync(Resolution, local, remote, ("id","name"))
        self.session.commit()

    def update_licenses(self):
        local = self.session.query(License).all()
        remote = self.connector.get_licenses()
        self.sync(License, local, remote, ("id","name","html","pdf"))
        self.session.commit()

    def update_categories(self):
        local = self.session.query(Category).all()
        remote = self.connector.get_types()
        self.sync(Category, local, remote, ("id","name"))
        self.session.commit()

    def update_forced_seeds(self):
        remote = self.connector.get_forced_seeds()
        r_dict = {}
        for r in remote:
            r_dict[int(r['oeuvre_id'])] = dt.datetime.fromtimestamp(int(r['seed_until_ts']))
        for l in self.session.query(Movie).filter(
                Movie.id.in_(r_dict.keys())).filter(Movie.download_completion == 100).all():
            try:
                if l.seed_until < r_dict[l.id]:
                    l.seed_until = r_dict[l.id]
                    self.session.add(l)
                    self.agent.enqueue(l)
                    self.session.commit()
            except TypeError:
                l.seed_until = dt.datetime.now()


    def update_movies(self):
        VEPLog("videoenpoche-agentd", "RevisionHandler: upgrade local repository %s"%self.local_revision)
        remotes = self.connector.get_upgrade_from_server(
                        self.local_revision)['movies']
        for remote in remotes:
            if remote.has_key("deleted") and remote["deleted"] is True:
                movie = self.session.query(Movie).get(remote["id"])
                if movie:
                    try:
                        self.agent.dequeue(movie)
                    except:
                        VEPLog(__name__, "Can't dequeue movie # %s" % remote['id'])
                    try:
                        self.package_handler.delete_movie(movie)
                    except:
                        VEPLog(__name__, "Can't delete movie # %s" % remote['id'])
            else:
                VEPLog(__name__, "Downloading package # %s" % remote['id'])
                self.handle_package(self.fetch_package(
                                            remote['package_url']),
                                            remote['id'])
            self.local_revision = remote['version']
            self.session.commit()

    def fetch_package(self, package_url):
        url = config.get("vep_server")[:-1] + package_url
        passwd_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passwd_mgr.add_password(None, config.get('vep_server'),
        config.get('login'), config.get('passwd'))
        handler = urllib2.HTTPBasicAuthHandler(passwd_mgr)
        opener = urllib2.build_opener(handler)
        VEPLog("videoenpoche-agentd", "RevisionHandler: download package from %s"%url)
        # Extract the filename
        filename=url.split('/')[-1].split('#')[0].split('?')[0]
        # First we open the destination file
        path = os.path.join (config.get('datadir'),config.get('packages_pending_dir'),filename)
        dest = open(path, 'wb')
        # Write the file from the downloaded file
        dest.write(opener.open(url).read())
        # Let's close the file
        dest.close()
        return path

    def handle_package(self,package_path, movie_id):
        movie = self.session.query(Movie).get(movie_id)
        if movie:
            #Edition
            state, movie = self.package_handler.edit_movie(movie,
                                                           package_path,
                                                           self.agent.dequeue)
        else:
            state, movie = self.package_handler.new_movie(package_path)
        self.package_handler.currentize_fresh_package(package_path, movie)
        if state in ("new", "edit+torrent"):
            self.agent.enqueue(movie)


class TorrentsManager(object):

    def _get_torrent_for_movie(self, movie):
        pkg = self.package_handler.get_current_package_for_movie(movie)
        return pkg.get_torrent()

    def _initialize_lt_session(self):
        session = lt.session()
        session.listen_on(6881,6891)
        session.set_upload_rate_limit(int(config.get("max_upload_rate")))
        settings = session.settings()
        settings.file_checks_delay_per_block = 4
        session.set_settings(settings)
        self.lt_session = session

    def _initialize_movies_list(self):
        with self.movies_list_lock:
            #First we enqueue what we were downloading
            for m in self.session.query(Movie).filter(
                    Movie.state=="downloading").all():
                self.enqueue(m)
            for m in self.session.query(Movie).filter(
                        or_(Movie.download_completion < 100,
                            Movie.seed_until > dt.datetime.now())).all():
                self.enqueue(m)

    def __init__(self, agent):
        """At startup, TorrentManager initializes the queues, the session
        and start filling the downloading queue and the seeding queue"""
        self.movies_list_lock = threading.RLock()
        self.session = agent.session
        self.package_handler = Package_handler(self.session)
        self.downloading = []
        self.dl_queue = []
        self.seeding = []
        self._initialize_lt_session()
        self._initialize_movies_list()

    def run(self):
        with self.movies_list_lock:
            for m in self.downloading[:]:
                state = m.torrent_handle.status().state
                VEPLog(__name__,"M%s.state : %s" % (m.id, state))
                if m.torrent_handle.is_seed():
                    #Movie is downloaded
                    #Moving it to the seed list
                    self.downloading.remove(m)
                    VEPLog(__name__,"Start seeding movie # %s" % m.id)
                    self.seeding.append(m)
                    if m.state not in ("announced", "visible"):
                        #Movie is not available yet
                        m.state = "announced"
                        m.download_completion = 100
                    self.move_dl_queue()
                else:
                    state = m.torrent_handle.status()
                    VEPLog(__name__,"%s movie # %s %s%%"
                        %(state.state, m.id, state.progress*100))
                    if "%s" % state.state == "downloading":
                        m.download_completion = state.progress*100
                    if m.state not in ("announced", "visible"):
                        m.state = "downloading"
                    self.session.add(m)
                    self.session.commit()
            for m in self.seeding[:]:
                    VEPLog(__name__,"Seeding movie # %s" % m.id)
                    if m.seed_until < dt.datetime.now():
                        #Check if we have to keep seeding
                        self.dequeue(m)

    def move_dl_queue(self):
        """Move the movies in the download queue"""
        while len(self.downloading) < 6 and len(self.dl_queue) > 0:
            m = self.dl_queue[0]
            if m not in self.downloading:
                VEPLog(__name__,"Adding movie # %s to the torrent session" % m.id)
                self.downloading.append(m)
                m.torrent_handle = self.lt_session.add_torrent({"ti": m.torrent,
                                              'save_path': os.path.join(
                                               config.get('datadir'),"movies")})
            self.dl_queue.pop(0)

    def enqueue(self, movie):
        with self.movies_list_lock:
                torrent_str = self._get_torrent_for_movie(movie)
                movie.torrent = lt.torrent_info(lt.bdecode(torrent_str))
                if movie.download_completion >= 100:
                    self.seeding.append(movie)
                    movie.torrent_handle = self.lt_session.add_torrent(
                                           {"ti": movie.torrent,
                                          'save_path': os.path.join(
                                           config.get('datadir'),"movies")})
                else:
                    if movie not in self.dl_queue:
                        self.dl_queue.append(movie)
                        self.move_dl_queue()

    def dequeue(self, movie):
        #Remove from all queues
        with self.movies_list_lock:
            self.dl_queue[:] = [m for m in self.dl_queue if m.id != movie.id]
            n_list = []
            for m in self.downloading:
                if m.id != movie.id:
                    n_list.append(m)
                else:
                    self.lt_session.remove_torrent(m.torrent_handle)
            self.downloading[:] = n_list
            n_list = []
            for m in self.seeding:
                if m.id != movie.id:
                    n_list.append(m)
                else:
                    try:
                        self.lt_session.remove_torrent(m.torrent)
                    except:
                        pass
            self.seeding[:] = n_list
            self.move_dl_queue()


class AgentManager(threading.Thread):

    def __init__(self):
        super(AgentManager, self).__init__()
        self.daemon = True
        self.keep_running = True

    def quit(self):
        VEPLog(__name__, "Exiting agent")
        self.keep_running = False

    def run(self):
        self.session = Session()
        self.update_mgr = UpdatesManager(self)
        self.torrent_mgr = TorrentsManager(self)
        while self.keep_running:
            try:
                self.update_mgr.run()
            except Exception , e:
                VEPLog(__name__, "Error with the updatemanager")

            try:
                self.torrent_mgr.run()
            except:
                VEPLog(__name__, "Error with the torrentmanager")
            time.sleep(float(config.get("thread_timer")))

    def enqueue(self, movie):
        VEPLog(__name__, "Enqueuing movie %s" % movie.id)
        self.torrent_mgr.enqueue(movie)

    def dequeue(self, movie):
        VEPLog(__name__, "dequeuing movie %s" % movie.id)
        self.torrent_mgr.dequeue(movie)

