#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import time

from reportlab.platypus import Paragraph, SimpleDocTemplate, Spacer, Table, TableStyle, CondPageBreak, PageBreak
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import cm
from reportlab.lib import colors


import config
import movies
from VideoEnPoche.dal import Session, Bill, Bill_movies
import cgv
import datetime as dt

session = Session()

# Some variable to get in the config
CITY=config.get ('seller_city')
VENUE=config.get ('seller_venue')
ADDRESS=config.get ('seller_address')
SIRET=config.get ('seller_siret')
EUROVATNUMBER=config.get ('seller_eurovatnumber')

# Some initialisation
PAGE_WIDTH=A4[0]/cm
PAGE_HEIGHT=A4[1]/cm

textVEP = """L’accès aux œuvres listées ci-dessus est conditionné au respect des Conditions Générales d’Utilisation affichées dans la salle et copiées sur votre support de stockage en même temps que les œuvres.<br />Vous trouverez plus d'information sur les films que vous venez d'acheter et sur les autres que nous vous proposons sur le site <font color=\"blue\"><link href=\"http://www.videoenpoche.info\">www.videoenpoche.info</link><font>."""

textMatroska = "Les films que vous avez acheté sont proposés au format ouvert et libre Matroska. Vous trouverez plus d'informations sur ce format ainsi que des informations sur les lecteurs de salon et les logiciels capable de le lire sur la page wikipedia <font color=\"blue\"><link href=\"http://fr.wikipedia.org/wiki/Matroska\">http://fr.wikipedia.org/wiki/Matroska</link></font>."

textVLC = "Pour une lecture sur un ordinateur nous vous recommandons le logiciel libre VLC. C'est un lecteur multimédia multiplateformes, fonctionnant aussi bien sous Windows, Mac OS X, toutes les distributions GNU/Linux et de nombreuses autres plateformes. Il est distribué sous la licence libre GNU/GPL. Un des grands atouts de VLC est qu'il intègre les codecs nécessaires à la lecture de la plupart des formats audio et vidéo. C'est pourquoi nous le recommandons pour la lecture des films que vous avez acheté. Vous pouvez le télécharger gratuitement sur cette page <font color=\"blue\"><link href=\"http://www.videolan.org/vlc/\">http://www.videolan.org/vlc/</link></font>."

# Styles defintion
styles = getSampleStyleSheet()
styleN = styles['Normal']
styleT = styles['Title']

class PdfBill:
    def __init__ (self, movies_list):
        self.session = Session()
        self.movies = movies_list
        self.parts = []
        self.totals = { "priceVATExc":float("0"),
                        "VAT":float("0"),
                        "priceVATIncl":float("0") }
        self.today = time.strftime('%Y%m%d', time.localtime())
        try:
            bill_number = self.session.query(Bill.number).filter(
                Bill.number.like("%s-%%" % self.today)).\
                        order_by(Bill.id.desc()).first().number
        except AttributeError:
            bill_number = False
        if not bill_number:
            self.daily_bill_number = 1
        else:
            latest = int (bill_number.split('-')[1])
            self.daily_bill_number = latest + 1
        self.bill_number="%s-%03d"%(self.today,self.daily_bill_number)
        self.title="Facture "+self.bill_number
        self.bill_filename="FACT-"+self.bill_number+".pdf"
        self.full_filename = os.path.join (config.get('datadir'),
                                           'bills',
                                           self.bill_filename)

    def write (self):
        self._store_in_db ()
        self._gen_pdf ()

    def _store_in_db (self):
        bill = Bill(number=self.bill_number)
        self.session.add(bill)
        # we also temporarily store in files to manage uploads to the main
        # server later
        for key in self.movies.keys ():
            m = self.movies[key]
            bm = Bill_movies(movie_id=m.id, price=m.price, bill=bill,
                        vat=m.vat, right_holders_share=m.right_holders_share)
            self.session.add(bm)
        self.session.commit()

        #Write into the files
        bills_file = os.path.join (config.get ('datadir'),
                                   'transactions', 'bills-%s'%self.today)
        bills_movies_file = os.path.join (config.get ('datadir'),
                                        'transactions', 'bills_movies-%s'%self.today)
        open (bills_file, 'a').write ("bills_id:%d,timestamp:%s,number:%s\n"%
                                    (bill.id,bill.timestamp,bill.number))
        bm_fd = open (bills_movies_file, 'a')
        for bm in self.session.query(Bill_movies).\
                filter(Bill_movies.bill==bill).all():
            bm_fd.write (
                    "bills_id:%d,movies_id:%d,price:%.2f,vat:%.2f,pricevat:%.2f,iright_holders_share:%.2f\n"%
                (bill.id,m.id,m.price,m.vat,m.pricevat,
                    m.right_holders_share))

    def _gen_pdf (self):
        licenses = movies.get_licenses()
        self.parts.append(Spacer(0, 2*cm))
        headerBilled = []
        headerBilled.append(Paragraph("<b>Nom", styleN))
        headerBilled.append(Paragraph("<b>"+"Prix HT en €"+"</b>", styleN))
        headerBilled.append(Paragraph("<b>"+"TVA en €"+"</b>", styleN))
        headerBilled.append(Paragraph("<b>"+"Prix TTC en €"+"</b>", styleN))

        billedData = [headerBilled]

        for key in self.movies.keys ():
            m = self.movies[key]
            visa = ''
            for item in licenses:
                if item.id == m.license_id:
                    visa = item.pdf
                    break

            self.addItemsBilled(billedData, m.title,
                                            visa,
                                            m.price,
                                            m.vat,
                                            m.pricevat)

        colWidth=(12*cm,2*cm,2*cm,2*cm)
        itemsBilled=Table(billedData, colWidth)
        itemsBilled.setStyle(TableStyle([
              ('FONTSIZE', (0,0),(-1,-1),12),
              ('BOX', (0,0),(-1,-1), 1, colors.black),
              ('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),
              ('ALIGN', (1,1), (-1, -1), 'CENTER'),
              ('VALIGN', (0,0), (-1, -1), 'TOP')
              ]))
        self.parts.append(itemsBilled)

        self.parts.append(Spacer(0, 1*cm))

        self.totals["priceVATExc"] = "%.2f" % (float(self.totals["priceVATExc"]))
        self.totals["VAT"] = "%.2f" % (float(self.totals["VAT"]))
        self.totals["priceVATIncl"] = "%.2f" % (float(self.totals["priceVATIncl"]))

        totalData = []
        totalData.append([Paragraph("<b>Total HT en €</b>", styleN),
                                    self.totals["priceVATExc"]])
        totalData.append([Paragraph("<b>"+"TVA en €"+"</b>", styleN),
                                    self.totals["VAT"]])
        totalData.append([Paragraph("<b>Total TTC en €</b>", styleN),
                                    self.totals["priceVATIncl"]])

        colsWidth=(4*cm,4*cm)
        totalBilled=Table(totalData, colWidths=colsWidth)
        totalBilled.setStyle(TableStyle([
              ('FONTSIZE', (0,0),(-1,-1),12),
              ('BOX', (0,0),(-1,-1), 1, colors.black),
              ('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),
              ('ALIGN', (0,0), (-1, -1), 'CENTER'),
              ('VALIGN', (0,0), (-1, -1), 'TOP')
              ]))
        self.parts.append(totalBilled)

        self.parts.append(CondPageBreak(3*cm))
        self.parts.append(Spacer(0, 2*cm))
        self.parts.append(CondPageBreak(3*cm))
        self.parts.append(Paragraph(textVEP, styleN))
        self.parts.append(CondPageBreak(3*cm))
        self.parts.append(Paragraph(textMatroska, styleN))
        self.parts.append(CondPageBreak(3*cm))
        self.parts.append(Paragraph(textVLC, styleN))

        # General Conditions
        self.parts.append(PageBreak())
        self.parts.append(Paragraph("VIDEO EN POCHE<br />CONDITIONS GENERALES D’UTILISATION", styles['Heading2']))

        self.parts.append(Paragraph(cgv.textCGV_Presentation, styleN))
        self.parts.append(Paragraph(cgv.textCGV_Acceptation, styleN))
        self.parts.append(Paragraph(cgv.textCGV_Limites, styleN))
        self.parts.append(Paragraph(cgv.textCGV_Integrite, styleN))
        self.parts.append(Paragraph(cgv.textCGV_PreRequis, styleN))
        self.parts.append(Paragraph(cgv.textCGV_Reponsabilite, styleN))
        pdf = SimpleDocTemplate(self.full_filename, bottomMargin = 2.3*cm)
        pdf.build(self.parts,
                  onFirstPage=self.myFirstPage,
                  onLaterPages=self.myLaterPages)

    def myFirstPage(self, canvas, doc):
        canvas.saveState()

        # Top Logo
        if os.path.isfile ("VEP.jpg"):
            img = "VEP.jpg"
        else:
            img = "/usr/share/videoenpoche/VEP.jpg"
        canvas.drawImage(img, 2.5*cm, 25*cm, 18*cm, 3.3*cm)

        # Top Link
        link="http://www.videoenpoche.info"
        canvas.setFillColor(colors.blue)
        canvas.setFont('Times-Bold',16)
        canvas.drawCentredString(A4[0]/2, 25.5*cm, link)
        linkRect=((A4[0]/2)-3.6*cm, 25.9*cm,(A4[0]/2)+3.6*cm,25.4*cm)
        canvas.setFont('Times-Bold',10)
        canvas.linkURL("http://www.videoenpoche.info",linkRect,1,0)

        # Title
        canvas.setFont('Times-Bold',15)
        canvas.setFillColor(colors.black)
        canvas.drawCentredString(A4[0]/2, 24.5*cm, self.title)

        # Location and Date
        canvas.setFont('Times-Bold',13)
        lieudate = CITY+" le "+time.strftime('%d/%m/%y', time.localtime())+","
        canvas.drawString(14*cm, 23.7*cm, lieudate)

        # Footer
        canvas.setFont('Times-Roman',9)
        canvas.drawString(5.5*cm, 1.25 * cm, "%s - %s" % (VENUE, ADDRESS))
        canvas.drawString(4.8*cm, 0.75 * cm, "SIRET : %s - TVA INTRACOMMUNAUTAIRE : %s" % (SIRET, EUROVATNUMBER))

        # Let's continue from here
        canvas.translate(0*cm, -3*cm)

    def myLaterPages(self, canvas, doc):
        canvas.saveState()

        # Footer
        canvas.setFont('Times-Roman',9)
        canvas.drawString(5.5*cm, 1.25 * cm, "%s - %s" % (VENUE, ADDRESS))
        canvas.drawString(4.8*cm, 0.75 * cm, "SIRET : %s - TVA INTRACOMMUNAUTAIRE : %s" % (SIRET, EUROVATNUMBER))

        canvas.restoreState()

    def addItemsBilled(self, items, title, licenceNotice, priceVATExc, VAT, priceVATIncl):
        item = []
        if (title):
            item.append(Paragraph("<b>"+title+"</b>"+"<br />"+licenceNotice, styleN))
        else:
            item.append(Paragraph(licenceNotice, styleN))
        item.append("%.2f" % (float(priceVATExc)))
        item.append("%.2f" % (float(VAT)))
        item.append("%.2f" % (float(priceVATIncl)))

        # Add the various elements to our bill
        items.append(item)
        self.totals["priceVATExc"]=self.totals["priceVATExc"]+float(priceVATExc)
        self.totals["VAT"]=self.totals["VAT"]+float(VAT)
        self.totals["priceVATIncl"]=self.totals["priceVATIncl"]+float(priceVATIncl)



def get_bills(offset=0, limit=30):
    ldb = session.query(Bill).all()
    return format_bills_from_db(ldb)

def get_last_bills():
    #build timedelta
    delta = dt.timedelta(days=30)
    upper_limit = dt.datetime.now() - delta

    ldb = session.query(Bill).\
            filter(Bill.timestamp>=upper_limit).all()
    return format_bills_from_db(ldb)


def format_bills_from_db(ldb):
    ret = {}
    bills = {}
    for b in ldb:
        for bi in b.bill_movies:
            bill_item = {'id': bi.movie_id,
                         'price': round(bi.price,2),
                         'vat': round(bi.vat,2),
                         'right_holders_share': round(bi.right_holders_share,2),
                    }
            try:
                bills[b.id]['items'].append(bill_item)
            except KeyError:
                bills[b.id] = {"items": [bill_item,],
                               "sale_date" : b.timestamp.\
                                       strftime("%Y-%m-%d %H:%M:%S"),
                               "number": b.number,}
    return bills
