#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from configobj import ConfigObj

from urllib2 import HTTPPasswordMgrWithDefaultRealm, HTTPBasicAuthHandler, Request, HTTPError, build_opener

CONFIGFILE = "/etc/videoenpoche.cfg"

DEFAULT = {
    'datadir': "/var/lib/videoenpoche/run",
    'user': 'videoenpoche',
    'group': 'videoenpoche',

    'sql_engine': 'sqlite',

    'server_port': 8080,
    'web_theme': 'default',

    'admin_password_file': 'admin_password',
    
    'packages_pending_dir': 'packages/pending',
    'packages_current_dir': 'packages/current',
    
    'vep_server': 'http://admin.videoenpoche.info:8000/',
    'api_url': 'api',
    'tracker_url': 'http://tracker.videoenpoche.info/tracker.php',
    
    'locale': 'fr_FR.UTF-8',

    'login': '',
    'passwd': '',
    'seller_city': '',
    'seller_venue': '',
    'seller_address': '',
    'seller_siret': '',
    'seller_eurovatnumber': '',

    'thread_timer': 60.0,
    'startup_delay': 60.0,

    'max_upload_rate': 70000,
    'max_concurrent_downloads': 5,

    'enable_local_http': True,
    'enable_agent': True,

    'debug': False,
    'authenticated' : True
}

DEFAULT.update({'movies_dir': os.path.join (DEFAULT['datadir'], 'movies'),
                'packages_dir': os.path.join (DEFAULT['datadir'], 'packages'),
                'bills_url': "/".join([DEFAULT['vep_server'] + DEFAULT['api_url'], 'bills']),
                'add_oeuvres_url': (DEFAULT['vep_server'] + 'oeuvres/add/')}
              )


__config = ConfigObj (CONFIGFILE)

def get(key):
    if key in __config.keys():
        if __config[key] == "True" or __config[key] == "true":
            __config[key] = True
        if __config[key] == "False" or __config[key] == "false":
            __config[key] = False
        return __config[key]
    elif key in DEFAULT:
        return DEFAULT[key]
    return None

def reload_agent ():
    try:
        f = open (os.path.join (get('datadir'), 'agent.pid'), 'r')
        pid = int (f.read ())
        f.close ()
        os.kill (pid, 1) # send SIGHUP to make the agent reload
    except:
        pass

def set_admin_password (password):
    filename = os.path.join (get ('datadir'),
                             get ('admin_password_file'))
    open (filename, 'w').write (password)
    os.chmod (filename, 0600)

def update (a):
    continue_debug = False
    if get('debug') == True:
        continue_debug = False
        
    for k in a.keys ():
        if k == 'admin_password':
            set_admin_password (a[k])
            continue
        if k == 'debug':
            __config['debug']=True
            continue_debug = True
            continue
        if a[k]:
            __config[k] = a[k]
        else:
            __config[k] = DEFAULT[k]

    if (check_connection_config(get('login'), get('passwd')) == "OK"):
        __config['authenticated']=True
    else:
        __config['authenticated']=False

    if continue_debug == False:
        __config['debug']=False

    __config.write ()
    reload_agent()
    
def check_connection_config(login=None, passwd=None):
    if not login:
        login=__config.get('login')
    
    if not passwd:
        passwd=__config.get('passwd')

    #Get the Connection state
    passwd_mgr = HTTPPasswordMgrWithDefaultRealm()
    passwd_mgr.add_password(None, get('vep_server'), login, passwd)
    httphandler = HTTPBasicAuthHandler(passwd_mgr)
    opener = build_opener(httphandler)

    connection = "OK"
    url = "%s/upgrade/%s" % (get('vep_server')+get('api_url'),'0')  

    try :
        opener.open(Request(url))
    except HTTPError, e:
        connection = "Fail : "+str(e.code)

    return connection
