from sqlalchemy import create_engine
from sqlalchemy import Column, Integer, String, ForeignKey
from sqlalchemy import Text, Numeric, DateTime
from sqlalchemy.sql.expression import func
from sqlalchemy.orm import sessionmaker, relation
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm.attributes import InstrumentedAttribute
import decimal
from VideoEnPoche import config
import os


db_path = os.path.join(config.get("datadir"),"movies.db")
engine = create_engine('sqlite:///%s' % db_path, echo=False)
Base = declarative_base()


class Serializable(object):

    def prep_for_json(self):
        odict = {}
        for key, what in self.__class__.__dict__.items():
            if isinstance(what, InstrumentedAttribute):
                if not what.impl.uses_objects:
                    ser = self.__getattribute__(key)
                    if isinstance(ser, decimal.Decimal):
                        ser = "%.2f" % round(ser,2)
                    odict[key] = ser
        return odict


class Revision(Serializable, Base):
    __tablename__ = "revisions"
    number = Column(Integer, primary_key=True)


class License(Serializable, Base):
    __tablename__ = "licenses"
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)
    html = Column(Text)
    pdf = Column(Text)


class Resolution(Serializable, Base):
    __tablename__ = "resolutions"
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)


class Category(Serializable, Base):
    __tablename__ = "categories"
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)


class Movie(Serializable, Base):
    __tablename__ = "movies"
    id = Column(Integer, primary_key=True)
    title = Column(String, nullable=False)
    desc = Column(Text)
    bonus_desc = Column(Text)
    length = Column(Integer)
    price = Column(Numeric)
    vat = Column(Numeric)
    right_holders_share = Column(Numeric)
    poster_filename = Column(String)
    filename = Column(String, nullable=False)
    size = Column(Integer, default=0)
    visa = Column(String)


    added = Column(DateTime)
    release_date = Column(DateTime)
    add_date = Column(DateTime)
    seed_until = Column(DateTime)

    #states : ("visible", "downloading", "new", "announced")
    state = Column(String, default="new")
    download_completion = Column(Numeric, default=0.0)

    license_id = Column(Integer, ForeignKey("licenses.id"))
    license = relation(License, primaryjoin=license_id == License.id)

    resolution_id = Column(Integer, ForeignKey("resolutions.id"))
    resolution = relation(Resolution,
            primaryjoin=resolution_id == Resolution.id)

    category_id = Column(Integer, ForeignKey("categories.id"))
    category = relation(Category, primaryjoin=category_id == Category.id)

    @property
    def pricevat(self):
        return self.price + self.vat

class Bill(Serializable, Base):
    __tablename__ = "bills"
    id = Column(Integer, primary_key=True)
    timestamp = Column(DateTime, default=func.now())
    number = Column(String)
    bill_movies = relation("Bill_movies")


class Bill_movies(Serializable, Base):
    __tablename__ = "bills__movies"
    id = Column(Integer, primary_key=True)
    price = Column(Numeric)
    vat = Column(Numeric)
    right_holders_share = Column(Numeric)

    movie_id = Column(Integer, ForeignKey("movies.id"))
    movie = relation(Movie, primaryjoin=movie_id == Movie.id)

    bill_id = Column(Integer, ForeignKey("bills.id"))
    bill = relation(Bill, primaryjoin=bill_id == Bill.id)

    @property
    def pricevat(self):
        return self.price + self.vat

def create_db():
    Base.metadata.create_all(engine)



Session = sessionmaker(engine)
