#!/usr/bin/python
# -*- coding: utf-8 -*-
# 
# Copyright (C) 2010 Guillaume Espanel <guillaume.espanel@objectif-libre.com>
# Copyright (C) 2010 Christophe Sauthier 
# <christophe.sauthier@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import urllib2


class httpbasicauthhandler(urllib2.HTTPBasicAuthHandler):
    def __init__(self, *args, **kwargs):
        urllib2.HTTPBasicAuthHandler.__init__(self, *args, **kwargs)
        self.retried_req = None

    def reset_retry_count(self):
        # Python 2.6.5 will call this on 401 or 407 errors and thus loop
        # forever. We disable reset_retry_count completely and reset in
        # http_error_auth_reqed instead.
        pass

    def http_error_auth_reqed(self, auth_header, host, req, headers):
        # Reset the retry counter once for each request.
        if req is not self.retried_req:
            self.retried_req = req
            self.retried = 0
        return urllib2.HTTPBasicAuthHandler.http_error_auth_reqed(
                        self, auth_header, host, req, headers)


class JasonImporter(object):
    api_url = None
    conn = None
    headers = {"Content-type": "application/json", "Accept": "text/plain"}

    def __init__(self, api_url, login, passwd):
        self.api_url = api_url
        passwd_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        passwd_mgr.add_password(None, api_url, login, passwd)
        handler = httpbasicauthhandler(passwd_mgr)
        self.opener = urllib2.build_opener(handler)
        urllib2.install_opener(self.opener)


    def _get_json_from(self, url):
        return json.load(self.opener.open(urllib2.Request(url, None,
                                                        self.headers)))

    def get_crews(self):
        url = "%s/crews/" % self.api_url
        return self._get_json_from(url)

    def get_movies(self):
        url = "%s/oeuvres/" % self.api_url
        return self._get_json_from(url)

    def get_visas(self):
        url = "%s/visas/" % self.api_url
        return self._get_json_from(url)

    def get_upgrade_from_server(self, timestamp):
        url = "%s/upgrade/%s" % (self.api_url,timestamp)
        return self._get_json_from(url)

    def get_licenses(self):
        url = "%s/licenses/" % self.api_url
        return self._get_json_from(url)

    def get_rightsholders(self):
        url = "%s/rightsholders/" % self.api_url
        return self._get_json_from(url)

    def get_resolutions(self):
        url = "%s/resolutions/" % self.api_url
        return self._get_json_from(url)

    def get_types(self):
        url = "%s/categories/" % self.api_url
        return self._get_json_from(url)

    def get_forced_seeds(self):
        url = "%s/seed/" % self.api_url
        return self._get_json_from(url)

