from VideoEnPoche.log import VEPLog
from VideoEnPoche.packages import Package
from VideoEnPoche import config
import os
import shutil
import datetime as dt




class Package_handler(object):

    def __init__(self, session):
        self.session = session

    def get_current_package_for_movie(self, movie):
        pkg_path = os.path.join(config.get("packages_dir"),
                    "current", "package_%s.zip" % movie.id)
        return Package(package=pkg_path)

    def delete_movie(self, movie, before_callback=None):
        VEPLog(__name__, "will delete movie # %s" % movie.id)
        if before_callback:
            before_callback(movie)
        self.clear_files(movie)
        self.session.delete(movie)
        self.session.commit()

    def clear_files(self, movie):
        VEPLog(__name__, "Deleting files for movie # %s" % movie.id)
        rmed = []
        if movie.poster_filename:
            to_rm = os.path.join(config.get("datadir"), "default", "images",
                    movie.poster_filename)
            try:
                os.remove(to_rm)
                rmed.append(to_rm)
            except OSError, e:
                VEPLog("videoenpoche", "Error : Could not delete %s" % to_rm)
        if movie.filename:
            to_rm = os.path.join(config.get("datadir"), "movies",
                    movie.filename)
            try:
                shutil.rmtree(to_rm)
                rmed.append(to_rm)
            except OSError, e:
                VEPLog("videoenpoche", "Error : Could not delete %s" % to_rm)
        to_rm = os.path.join(config.get("datadir"),
                config.get("packages_current_dir"),
                "package_%s.zip" % movie.id)
        try:
            os.remove(to_rm)
            rmed.append(to_rm)
        except OSError, e:
            VEPLog("videoenpoche", "Error : Could not delete %s" % to_rm)
        VEPLog("videoenpoche", "Movies: Deletion of : %s done"%(", ".join(rmed)))

    def currentize_fresh_package(self, package_file, movie):
        shutil.move(package_file, os.path.join(config.get("packages_dir"),
                    "current", "package_%s.zip" % movie.id))

    def edit_movie(self, local_movie, package_file, before_delete_callback=None):
        package = Package(package=package_file)
        local_package = self.get_current_package_for_movie(local_movie)
        remote_movie = package.movie_from_metas()
        if local_package.torrent_hash != package.torrent_hash:
            #New torrent
            VEPLog(__name__, "New torrent for movie #%s" % local_movie.id)
            self.delete_movie(local_movie, before_delete_callback)
            op = "edit+torrent"
        else:
            #Not a new torrent
            VEPLog(__name__, "Keeping torrent for movie #%s" % local_movie.id)
            remote_movie.download_completion = local_movie.download_completion
            remote_movie.seed_until = local_movie.seed_until
            remote_movie.state = local_movie.state
            self.session.delete(local_movie)
            op = "edit"
        package.extract_poster()
        self.session.add(remote_movie)
        return (op, remote_movie)

    def new_movie(self, package_file):
        #Save the poster
        package = Package(package=package_file)
        movie = package.movie_from_metas()
        movie.seed_until = dt.datetime.now() + dt.timedelta(days=20)
        movie.download_completion = 0.0
        self.session.add(movie)
        package.extract_poster()
        self.session.commit()
        VEPLog(__name__, "Movie %s has been created" % (movie.id))
        return "new", movie

