from VideoEnPoche.dal import Movie
from VideoEnPoche import config
from datetime import datetime


import urllib2
import json
import libtorrent as lt
import zipfile
import os
import mechanize
import hashlib

from log import VEPLog


class Package(object):
    trackers = None
    package_path = None
    movie_path = None
    bonus_path = None
    movie = None

    def __init__(self, movie=None, package=None):
        if package is None:
            self.movie = movie
            self.trackers = []
            self.trackers.append(config.get('tracker_url'))
            self.movie_path = os.path.join(config.get('movies_dir'),
                                          self.movie.filename)
            self.package_path = os.path.join(config.get('packages_dir'),
                                          self.movie.filename+".zip")
        else:
            self.package_path = package

    def make(self):
        fs = lt.file_storage()
        if lt.version_minor < 15 and lt.version_major == 0:
            lt.add_files(fs,self.movie_path)
        else:
            lt.add_files(fs,self.movie_path,0)
        torrent = lt.create_torrent(fs)
        for tracker in self.trackers:
            torrent.add_tracker(tracker,0)
        torrent.set_creator("Video En Poche")
        lt.set_piece_hashes(torrent, config.get('movies_dir'))
        torrent_gen=torrent.generate()
        package = zipfile.ZipFile(self.package_path, "w")
        package.writestr("torrent",
                         lt.bencode(torrent_gen))
        movie_dict = self.movie.prep_for_json()
        movie_dict.update({"dist_network_id":self.movie.id})
        package.writestr("metadata", json.dumps(movie_dict))
        package.write(os.path.join(config.get("datadir"),
                                    "default",
                                    "images",
                                    self.movie.poster_filename),"poster")
        package.close()



    def post(self):
        VEPLog("videoenpoche-agentd", "AgentManager: Post package for %s" %
                self.movie.title)
        url = config.get('add_oeuvres_url')
        response = urllib2.urlopen(url)
        forms = mechanize.ParseResponse(response, backwards_compat=False)
        response.close()
        form = forms[0]
        form.add_file(open(self.package_path), "application/zip",
                self.movie.filename+".zip")
        request2 = form.click()
        response2 = urllib2.urlopen(request2)


    def get_metas(self):
        print "PATH : %s" % self.package_path
        package = zipfile.ZipFile(self.package_path, 'r')
        metas = json.load(package.open("metadata"))
        package.close()
        return metas

    def extract_poster(self):
        movie = self.get_metas()
        package = zipfile.ZipFile(self.package_path, 'r')
        package.extract(movie['poster_filename'],
                os.path.join(config.get('datadir'),
                    "default",
                    "images"))

    def movie_from_metas(self):
        movie = self.get_metas()
        fields = {
            'id': movie['dist_network_id'],
            'title': movie['title'],
            'desc': movie['desc'],
            'length': movie['length'],
            'price': movie['price'],
            'vat': movie['vat'],
            'right_holders_share': movie['right_holders_share'],
            'size': int(movie['filesize']),
            'filename': movie['filename'],
            'resolution_id': movie['resolutions_id'],
            'category_id': movie['categories_id'],
            'poster_filename': movie['poster_filename'],
            'add_date': datetime.strptime(movie["add_date"], '%Y-%m-%d'),
            'release_date': datetime.strptime(movie["release_date"], '%Y-%m-%d'),
            'license_id': movie['licenses_id'],
            'visa': movie['visa'],
            'state': "queued",
        }
        return Movie(**fields)

    @property
    def torrent_hash(self):
        package = zipfile.ZipFile(self.package_path, 'r')
        torrent = package.open("torrent")
        t_hash = hashlib.sha1(torrent.read()).digest()
        package.close()
        return t_hash

    def get_torrent(self):
        package= zipfile.ZipFile(self.package_path, 'r')
        torrent = package.open("torrent").read()
        package.close()
        return torrent


def get_pkg_from_name(pkg_name):
    pkg_dir = os.path.join (config.get('datadir'),config.get('packages_current_dir'))
    pkg_file = os.path.join(pkg_dir, pkg_name)
    return Package(movie=None, package=pkg_file)



def get_pkg_list():
    ms = {}
    pkg_dir = os.path.join (config.get('datadir'),config.get('packages_current_dir'))
    for root, dirs, files in os.walk(pkg_dir, topdown=False):
        for name in files:
            pkg_file = os.path.join(pkg_dir, name)
            pkg = Package(movie=None, package=pkg_file)
            ms[pkg.movie_from_metas().title] = name
    return ms
