#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import glob
import gio

import config
import bills

class Transaction:
    def __init__ (self, device, movies):
        self.device = device
        self.movies = movies
        self.device.transaction = self
        self.to_copy = []

        for m in movies.keys ():
            dirname = os.path.join (config.get ('datadir'),
                                    'movies',
                                    movies[m].filename);
            files = glob.glob ('%s/*'%dirname)
            for f in files:
                self.to_copy.append (f)

        if self.to_copy:
            device.multiple_copy (self.to_copy)


    def cancel (self):
        self.device.cancel_transaction ()

    def ack_cancel (self):
        self.device.ack_cancel ()

    def conclude (self):
        bill = bills.PdfBill(self.movies)
        bill.write ()

        # copy the bill on the USB key
        target = os.path.join (self.device.mountpoint, bill.bill_filename)
        source = bill.full_filename
        src_file = gio.File (path=source)
        dst_file = gio.File (path=target)
        src_file.copy (dst_file, flags=gio.FILE_COPY_OVERWRITE)
