#!/usr/bin/python
# -*- coding: utf-8 -*-
from sqlalchemy import create_engine
from sqlalchemy import Column, Integer, String, MetaData
from sqlalchemy import Table
from sqlalchemy.orm import sessionmaker
from VideoEnPoche import config
import os

from VideoEnPoche.dal import Session, Movie, Revision, License, Resolution
from VideoEnPoche.dal import Category, Bill, Bill_movies, create_db
from datetime import datetime as dt
import shutil
import sys

old_db_path = os.path.join(config.get("datadir"),"movies.old.db")
try:
    shutil.move(os.path.join(config.get("datadir"),"movies.db"), old_db_path)
except:
    sys.exit(0)
old_engine = create_engine('sqlite:///%s' % old_db_path, echo=False)
old_meta = MetaData()
old_meta.reflect(bind=old_engine)
oses = sessionmaker(old_engine)()

#Creation nouvelle db
create_db()
nses = Session()

#Replication des License
olicenses = old_meta.tables["licenses"]
for l in oses.query(olicenses).all():
    nl = License(id=l.licenses_id, name=l.name, pdf=l.pdf, html=l.html)
    nses.add(nl)
nses.commit()

#replication resolutions
oresolutions = old_meta.tables["resolutions"]
for r in oses.query(oresolutions).all():
    nr = Resolution(id=r.resolutions_id, name=r.name)
    nses.add(nr)
nses.commit()

#replication categories
otypes = old_meta.tables["types"]
for t in oses.query(otypes).all():
    nc = Category(id=t.types_id, name=t.name)
    nses.add(nc)
nses.commit()

#replication bills
obills = old_meta.tables["bills"]
for b in oses.query(obills).all():
    nb = Bill(id=b.bills_id, timestamp=b.timestamp,
            number=b.number)
    nses.add(nb)
nses.commit()

#replication movies
omovies = Table('movies', old_meta,
        Column("desc", String(length=10000)),
        Column("seed_until", Integer),useexisting=True)
for m in oses.query(omovies).all():
    if not m.seed_until:
        seed_until = 2
    else:
        seed_until = m.seed_until

    if not m.add_date:
        add_date = dt.fromtimestamp(2)
    else:
        add_date = dt.strptime(m.add_date, "%Y-%m-%d")

    if not m.release_date:
        release_date= dt.fromtimestamp(2)
    else:
        release_date= dt.strptime(m.release_date, "%Y-%m-%d")

    nm = Movie(id=m.movies_id, title=m.title, desc=m.desc,
            bonus_desc=m.bonus_desc, length=m.length, price=m.price,
            vat=m.vat, right_holders_share=m.right_holders_share,
            poster_filename=m.poster_filename, filename=m.filename,
            size=m.filesize, visa=m.visa, added=m.added,
            release_date=release_date, add_date=add_date,
            seed_until=dt.fromtimestamp(float(seed_until)), state=m.state,
            download_completion=m.download_completion,
            license_id=m.license, resolution_id=m.resolution,
            category_id=m.type)
    nses.add(nm)
nses.commit()

#replication bills movies
obms = old_meta.tables["bills_movies"]
for bm in oses.query(obms).all():
    nbm = Bill_movies(price=bm.price, vat=bm.vat,
            right_holders_share=bm.right_holders_share, movie_id=bm.movies_id,
            bill_id=bm.bills_id)
    nses.add(nbm)
nses.commit()

#replication revisions
orevs = old_meta.tables["revision"]
for orev in oses.query(orevs).distinct():
    nr = Revision(number=orev.number)
    nses.add(nr)
nses.commit()

