#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import dbus
import simplejson as json

import config
import movies
import packages

clients = []

def _serialize (obj):
    try:
        return obj.prep_for_json ()
    except AttributeError:
        return "%s" % obj

def _send_json (handler, json):
    handler.send_response (200)
    handler.send_header ('Content-Type', 'application/json')
    handler.end_headers ()
    handler.wfile.write (json)

def _send_text (handler, text):
    handler.send_response (200)
    handler.send_header ('Content-Type', 'text/plain')
    handler.end_headers ()
    handler.wfile.write (text)

def ws_get_settings (handler):
    a = {}
    for k in ['login', 'passwd',
              'seller_city', 'seller_venue', 'seller_address',
              'seller_siret', 'seller_eurovatnumber', 'debug']:
        a[k] = config.get(k)
    data = json.dumps (a, skipkeys=True)
    _send_json (handler, data)

def ws_get_movies_number (handler):
    spl = handler.path.split ('/')
    method = spl[3]
    try:
        search_str = spl[4]
    except:
        search_str = None
    _send_text (handler, movies.get_number (method, search_str))

def ws_get_movies (handler):
    spl = handler.path.split ('/')
    if len (spl) < 4:
        list = movies.get_movies_by_date ()
    else:
        if spl[3] == 'default':
            try:
                offset = int (spl[4])
            except:
                offset = 0
            list = movies.get_movies_by_date (offset)
        elif spl[3] == 'type':
            try:
                offset = int (spl[5])
            except:
                offset = 0
            type_id = int (spl[4])
            list = movies.get_movies_by_type (type_id, offset)
        elif spl[3] == 'search':
            try:
                offset = int (spl[5])
            except:
                offset = 0
            list = movies.get_movies_from_search (spl[4], offset)
        elif spl[3] == 'state':
            list= movies.get_movies_by_state(spl[4])
        else:
            list = movies.get_movies_by_date ()

    data = json.dumps (list, skipkeys=True, default=_serialize)
    _send_json (handler, data)

def ws_get_types (handler):
    list = movies.get_types ()
    data = json.dumps (list, skipkeys=True, default=_serialize)
    _send_json (handler, data)

def ws_get_resolutions (handler):
    list = movies.get_resolutions ()
    data = json.dumps (list, skipkeys=True, default=_serialize)
    _send_json (handler, data)

def ws_get_licenses(handler):
    list = movies.get_licenses()
    data = json.dumps (list, skipkeys=True, default=_serialize)
    _send_json (handler, data)

def _get_proxy ():
    bus = dbus.SystemBus ()
    return bus.get_object ('info.videoenpoche.server', '/Devices')

def ws_get_all_devices (handler):
    proxy = _get_proxy ()
    devices = proxy.GetDevices ()
    _send_json (handler, json.dumps (devices, skipkeys=True))

def ws_get_available_devices (handler):
    proxy = _get_proxy ()
    devices = proxy.GetAvailableDevices ()
    print json.dumps (devices, skipkeys=True)

def ws_remove_device (handler):
    device = handler.path.split ('/')[-1]
    proxy = _get_proxy ()
    ret = proxy.RemoveDevice (device)

def ws_init_transaction (handler):
    spl = handler.path.split ('/')
    device_id = spl[3]
    movies = spl[4:]

    proxy = _get_proxy ()
    proxy.InitTransaction (device_id, movies)

def ws_cancel_transaction (handler):
    device = handler.path.split ('/')[-1]
    proxy = _get_proxy ()
    proxy.CancelTransaction (device)

def ws_ack_cancel (handler):
    device = handler.path.split ('/')[-1]
    proxy = _get_proxy ()
    proxy.AckCancel (device)

def ws_conclude_transaction (handler):
    device = handler.path.split ('/')[-1]
    proxy = _get_proxy ()
    proxy.ConcludeTransaction (device)

def ws_check_password (handler):
    given_password = handler.path.split ('/')[-1]
    filename = os.path.join (config.get ('datadir'),
                             config.get ('admin_password_file'))
    password = open (filename, 'r').readline ().strip()
    if password == given_password:
        _send_text (handler, 'true')
    else:
        _send_text (handler, 'false')

def ws_set_devices_handled (handler):
    b = handler.path.split ('/')[-1]
    proxy = _get_proxy ()
    proxy.SetDevicesHandled (b)

def ws_get_devices_handled (handler):
    proxy = _get_proxy ()
    if proxy.GetDevicesHandled ():
        _send_text (handler, 'true')
    else:
        _send_text (handler, 'false')

def ws_register_client (handler):
    if clients:
        new_id = clients[-1] + 1
    else:
        new_id = 1
    clients.append (new_id)
    # automatically activate the devices handling when the first client starts
    if len (clients) == 1:
        print "First client started: handling devices"
        proxy = _get_proxy ()
        proxy.SetDevicesHandled ("1")
    _send_text (handler, '%d'%new_id)

def ws_unregister_client (handler):
    id = int (handler.path.split ('/')[-1])
    try:
        clients.remove (id)
    except:
        pass
    # if the client is the last, stop handling the devices
    if not clients:
        print "No more clients: stop handling devices"
        proxy = _get_proxy ()
        proxy.SetDevicesHandled (False)
    _send_text (handler, '')

def ws_ack_announce (handler):
    spl = handler.path.split ('/')
    if spl[3] == 'movies':
      try:
        movie_id = spl[4]
      except:
        movie_id = 0
      movies.ack_announcement(movie_id)

def ws_check_debug (handler):
    if (config.get ('debug')):
        _send_text (handler, True)
    else:
        _send_text (handler, False)

def ws_get_debug_values (handler):
    from agentmanager import  RevisionHandler


    rh = RevisionHandler(session=False, checkonly=True)
    a = {}

    #Get the revision
    a['revision']=rh.get_local_timestamp_revision()

    #Get the Connection state
    a['connection'] = config.check_connection_config()

    data = json.dumps (a, skipkeys=True)
    _send_json (handler, data)

def ws_get_pkg_list(handler):
    pkgs = packages.get_pkg_list()
    data = json.dumps (pkgs)
    _send_json (handler, data)


def ws_get_pkg(handler):
    pkg_name = handler.path.split('/')[-1]
    pkg = packages.get_pkg_from_name(pkg_name).get_metas()
    data = json.dumps (pkg)
    _send_json (handler, data)
