#! /bin/bash
#
# Copyright (C) 2010 Guillaume Espanel <guillaume.espanel@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


#Usage :
# patcher.sh path_to_configy.py [update_schema]
# update_schema is used to write the schema to the patch files


CONFIG=$1
UPDATE_SCHEMA=$2

REVISION_TABLE="db_version"
UPGRADES_DIR=`dirname $0`"/upgrades"

#Get database file from 
DB_FILE=`echo "print get('datadir')" | python -i $CONFIG 2> /dev/null`
DB_FILE="${DB_FILE}/movies.db"

list_tables() {
	tables=`echo ".tables" | sqlite3 $DB_FILE`
}

get_patch_list() {
	next_rev=`echo "SELECT (version + 1)  FROM $REVISION_TABLE;" | sqlite3 $DB_FILE`
	max_rev=`ls $UPGRADES_DIR | sort -n | tail -n 1 | cut -d "." -f 1`
	steps=`seq $next_rev $max_rev`
}

apply_patch() {
	echo "Trying to apply $1"
	patch=$UPGRADES_DIR/$1.sql
	if [ ! -r "$patch" ]
	then
		echo Patch not found
		exit 1
	fi
	cat $patch | sqlite3 $DB_FILE
	if [ $? -ne 0 ]
	then
		echo Failed to apply patch
		exit 2
	fi
	echo "UPDATE $REVISION_TABLE SET version='$1';" | sqlite3 $DB_FILE
	if [ $? -ne 0 ]
	then
		echo Failed to change version number
		exit 3
	fi
	if [ -n $UPDATE_SCHEMA ]
	then
		write_schema $patch
	fi
}


apply_schema() {
	echo "Trying to apply schema $1"
	patch=$UPGRADES_DIR/$1.sql
	if [ ! -r "$patch" ]
	then
		echo Patch not found
		exit 1
	fi
	#Schema extraction
	LINE=`grep -n '\-\- Schema' $patch | cut -f1 -d \:`
	LINE=$(($LINE + 1))
	cat $patch | tail -n +$LINE | fgrep -v '/*' | fgrep -v '*/' | sqlite3 $DB_FILE
	echo "INSERT INTO $REVISION_TABLE VALUES('$1');" | sqlite3 $DB_FILE
	if [ $? -ne 0 ]
	then
		echo Failed to change version number
		exit 3
	fi
}


write_schema() {
	cat $patch | grep '^-- Schema : $' && return
	#Write schema if it's not alredy there
	echo "" >> $patch
	echo "" >> $patch
	echo "-- Schema : " >> $patch
	echo "/*" >> $patch
	echo ".schema" | sqlite3 $DB_FILE >> $patch
	echo "*/" >> $patch
}


main() {
	if [ ! -w $DB_FILE ]
	then
		echo "Database doesn't exist"
		echo "Creating database"
		max_rev=`ls $UPGRADES_DIR | sort -n | tail -n 1 | cut -d "." -f 1`
		apply_schema $max_rev
		exit 0
	fi
	list_tables
	#Get revision_table
	echo $tables | grep -o "$REVISION_TABLE" > /dev/null
	if [ $? -ne 0 ]
	then
		echo "Revision table doesn't exist"
		apply_patch 0
	fi
	get_patch_list
	for patch in $steps
	do
		apply_patch $patch
	done
}

main
