#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Gauvain Pocentek <gauvain.pocentek@objectif-libre.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from distutils.core import setup
from distutils.command.build import build
from distutils.dep_util import newer
from distutils.command.install_data import install_data

import os
import glob
import subprocess

class PoBuild (build):
  def run (self):
    build.run (self)

    for po in glob.glob (os.path.join ('po/', '*.po')):
      lang = os.path.basename(po).split('.')[0]
      mo = os.path.join('build', 'mo', lang, 'videoenpoche.mo')

      directory = os.path.dirname(mo)
      if not os.path.exists(directory):
        print('creating %s' % directory)
        os.makedirs(directory)

      if newer (po, mo):
        print('compiling %s -> %s' % (po, mo))
        try:
          rc = subprocess.call(['msgfmt', '-o', mo, po])
          if rc != 0:
            raise Warning, "msgfmt returned %d" % rc
        except Exception, e:
          print ("Building gettext files failed.")
          print ("Error: %s" % str(e))
          sys.exit(1)

class InstallData (install_data):
  def run (self):
    for mo in glob.glob (os.path.join ('build', 'mo', '*', 'videoenpoche.mo')):
      lang = os.path.basename(os.path.dirname(mo))
      dest = os.path.join('share', 'locale', lang, 'LC_MESSAGES')
      self.data_files.extend ([(dest, [mo])])
    install_data.run (self)

setup (name = "videoenpoche",
       version = "1.2.0",
       description = 'Vidéo En Poche',
       author = 'Objectif Libre',
       author_email = 'videoenpoche-dev@lists.launchpad.net',
       url = 'https://launchpad.net/videoenpoche',
       packages = ['VideoEnPoche'],
       scripts = ['videoenpoched', 'videoenpoche-upload_bills_to_server', 'videoenpoche-client'],
       data_files = [('/etc/dbus-1/system.d',
                        ['conf/info.videoenpoche.server.conf']),
                     ('/var/lib/polkit-1/localauthority/20-org.d',
                        ['conf/info.videoenpoche.server.pkla']),
                     ('/usr/share/videoenpoche',
                        ['VEP.jpg', 'VEP-icone.svg']),
                     ('/usr/share/applications',
                        ['videoenpoche.desktop'])
                    ],
       cmdclass={'build': PoBuild, 'install_data': InstallData}
       )
